<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema" 
  xmlns:zw="http://icebearsoft.euweb.cz"
  exclude-result-prefixes="xs zw"
  version="2.0">
  <!-- Zdeněk Wagner, http://icebearsoft.euweb.cz, Creative Commons: CC BY-SA -->
  
  <xsl:output method="text" encoding="UTF-8" indent="no"/>
  
  <xsl:strip-space elements="*"/>
  
  <xsl:function name="zw:twodigits" as="xs:string">
    <xsl:param name="num" as="xs:decimal"/>
    <xsl:value-of select="format-number($num, '00')"/>
  </xsl:function>
  
  <xsl:function name="zw:TeXdur" as="xs:string">
    <xsl:param name="dur" as="xs:duration"/>
    <xsl:variable name="H" as="xs:decimal" select="hours-from-duration($dur)"/>
    <xsl:variable name="M" as="xs:decimal" select="minutes-from-duration($dur)"/>
    <xsl:variable name="S" as="xs:decimal" select="seconds-from-duration($dur)"/>
    <xsl:value-of select="if ($H gt 0) then concat($H, '\C', zw:twodigits($M), '\C', zw:twodigits($S))
      else concat($M, '\C', zw:twodigits($S))"/>
  </xsl:function>
  
  <xsl:function name="zw:duration" as="xs:dayTimeDuration*">
    <xsl:param name="val"/>
    <xsl:sequence select="for $a in $val return $a cast as xs:dayTimeDuration"/>
  </xsl:function>
  
  <xsl:template match="/">
    <xsl:text>% Transformed by $Id: patkatha2latex.xsl 61 2018-11-16 23:22:43Z zw $&#10;</xsl:text>
    <xsl:text>\input utf8-t1&#10;</xsl:text>
    <xsl:text>\documentclass[12pt]{article}&#10;</xsl:text>
    <xsl:text>\usepackage{patkatha}&#10;</xsl:text>
    <xsl:text>\begin{document}&#10;</xsl:text>
    <xsl:apply-templates/>
    <xsl:if test="scénář/scéna/@délka">
      <xsl:value-of select="concat('&#10;\addtocontents{toc}{\Celkem{',
        zw:TeXdur(sum(zw:duration(/scénář/scéna/@délka))), '}}&#10;')"/>
    </xsl:if>
    <xsl:text>&#10;\tableofcontents&#10;</xsl:text>
    <xsl:text>\end{document}&#10;</xsl:text>
  </xsl:template>
  
  <xsl:template match="scénář">
    <xsl:value-of select="concat('\title{', @název, '}&#10;')"/>
    <xsl:value-of select="concat('\author{', @autor, '}&#10;')"/>
    <xsl:value-of select="concat('\svnId{', @svnId, '}&#10;')"/>
    <xsl:if test="@autor-hudby">
      <xsl:value-of select="concat('\composer{', @autor-hudby, '}&#10;')"/>
    </xsl:if>
    <xsl:if test="podtitul">
      <xsl:text>\subtitle{</xsl:text>
      <xsl:apply-templates select="podtitul"/>
      <xsl:text>}&#10;</xsl:text>
    </xsl:if>
    <xsl:text>\maketitle&#10;</xsl:text>
    <xsl:apply-templates select="* except podtitul"/>
  </xsl:template>
  
  <xsl:template match="podtitul">
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="seznam-osob">
    <xsl:text>&#10;\SeznamOsob&#10;</xsl:text>
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="seznam-osob/komentář|osoby/komentář">
    <xsl:if test="position() != 1">
      <xsl:text>\par\vb&#10;</xsl:text>
    </xsl:if>
    <xsl:apply-templates/>
    <xsl:text>&#10;</xsl:text>
    <xsl:if test="position() != last()">
      <xsl:text>\par\vb&#10;</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="osoba">
    <xsl:value-of select="concat('\osoba{', @jméno, '}')"/>
    <xsl:if test=". != ''">
      <xsl:text>\ZWcolon </xsl:text>
      <xsl:apply-templates/>
    </xsl:if>
    <xsl:variable name="jméno" as="xs:string" select="@jméno"/>
    <xsl:variable name="seznam" as="xs:integer*">
      <xsl:for-each select="/scénář/scéna">
        <xsl:if test="osoby/osoba/@jméno = $jméno">
          <xsl:value-of select="position()"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>
    <xsl:if test="not(empty($seznam))">
      <xsl:text> [</xsl:text>
      <xsl:value-of select="$seznam" separator=", "/>
      <xsl:text>]</xsl:text>
    </xsl:if>
    <xsl:text>&#10;&#10;</xsl:text>
  </xsl:template>
  
  <xsl:template match="scéna">
    <xsl:text>&#10;\scena</xsl:text>
    <xsl:if test="@délka">
      <xsl:value-of select="concat('[', zw:TeXdur(@délka cast as xs:duration), ']')"/>
    </xsl:if>
    <xsl:value-of select="concat('{', @název, '}&#10;')"/>
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="popis">
    <xsl:text>\begin{popis}&#10;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>&#10;\end{popis}&#10;&#10;</xsl:text>
  </xsl:template>
  
  <xsl:template match="osoby">
    <xsl:text>\osoby{</xsl:text>
    <xsl:apply-templates select="* except komentář"/>
    <xsl:text>}&#10;&#10;</xsl:text>
    <xsl:apply-templates select="komentář"/>
    <xsl:if test="komentář">
      <xsl:text>&#10;&#10;\vb&#10;</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="osoby/osoba">
    <xsl:if test="position() gt 1">
      <xsl:text>, </xsl:text>
    </xsl:if>
    <xsl:value-of select="concat('\osoba{', @jméno, '}')"/>
    <xsl:if test="not(//seznam-osob/osoba[@jméno = current()/@jméno])">
      <xsl:text> (chybí v seznamu osob)</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="píseň">
    <xsl:value-of select="concat('&#10;\begin{song}{', @název, '}&#10;')"/>
    <xsl:apply-templates/>
    <xsl:text>\end{song}&#10;&#10;</xsl:text>
  </xsl:template>
  
  <xsl:template match="refrén">
    <xsl:text>\Ref&#10;</xsl:text>
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="sloka">
    <xsl:text>\begin{sloka}&#10;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>\end{sloka}&#10;</xsl:text>
  </xsl:template>
  
  <xsl:template match="verš">
    <xsl:text>\vers</xsl:text>
    <xsl:if test="@osoba or @komentář">
      <xsl:text>[</xsl:text>
      <xsl:if test="@osoba">
        <xsl:value-of select="concat('osoba={', @osoba, '}')"/>
        <xsl:if test="@komentář">
          <xsl:text>,</xsl:text>
        </xsl:if>
      </xsl:if>
      <xsl:if test="@komentář">
        <xsl:value-of select="concat('komentar={', @komentář, '}')"/>
      </xsl:if>
      <xsl:text>]</xsl:text>
    </xsl:if>
    <xsl:value-of select="concat('{', ., '}&#10;')"/>
  </xsl:template>
  
  <xsl:template match="komentář">
    <xsl:text>\komentar{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
    <xsl:if test="local-name(parent::*) = ('scéna', 'píseň', 'refrén')">
      <xsl:text>&#10;</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="krát">
    <xsl:text> \krat </xsl:text>
  </xsl:template>
  
  <xsl:template match="důraz">
    <xsl:text>\duraz{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
  </xsl:template>
  
  <xsl:template match="sub">
    <xsl:text>\ensurenath{_{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}}</xsl:text>
  </xsl:template>
  
  <xsl:template match="řeč">
    <xsl:value-of select="concat('&#10;\osoba{', @osoba, '}')"/>
    <xsl:choose>
      <xsl:when test="local-name(child::*[1]) eq 'komentář'">
        <xsl:text>\ZWdelayedcolon </xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\ZWcolon </xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates/>
    <xsl:text>\par&#10;</xsl:text>
  </xsl:template>
  
  <xsl:template match="text">
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="text()">
    <xsl:value-of select="replace(replace(., '([ (,][AaIiKkOoSsUuVvZz])[ &#x0d;&#x0a;]+', '$1~'),
      '(~[AaIiKkOoSsUuVvZz])[ &#x0d;&#x0a;]+', '$1~')"/>
  </xsl:template>
  
  <xsl:template match="*"/>
</xsl:stylesheet>
